/**
 *
 * Project: Eight Track, File: UploadHandler.java
 *
 * Copyright 2019-2020 Locusworks LLC.
 * All rights reserved. Federal copyright law prohibits unauthorized reproduction by
 * any means and imposes fines up to $25,000 for violation.  No part of this material
 * may be reproduced, transmitted, transcribed, stored in a retrieval system, copied,
 * modified, duplicated, adapted or translated into another program language in any
 * form or by any means, electronic, mechanical, photocopying, recording, or
 * otherwise, without the prior written permission from Locusworks. Locusworks
 * affirms that Eight-Track(R) software and data is subject to United States
 * Government Purpose Rights.  Contact Locusworks, 1313 Lawnview Drive
 * Forney TX 75126, (802) 488-0438, for commercial licensing opportunities.
 *
 * IN NO EVENT SHALL LOCUSWORKS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LOCUSWORKS HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  NO RESPONSIBILITY IS ASSUMED BY
 * LOCUSWORKS FOR ITS USE, OR FOR ANY INFRINGEMENTS OF PATENTS OR OTHER RIGHTS OF
 * THIRD PARTIES RESULTING FROM ITS USE.  LOCUSWORKS SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND
 * ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS
 * IS". LOCUSWORKS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 */
package net.locusworks.discord.eighttrack.events.main.handlers;

import java.awt.Color;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.time.OffsetDateTime;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.Message.Attachment;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.locusworks.crypto.utils.HashUtils;
import net.locusworks.discord.eighttrack.audio.Mp3UploadHandler;
import net.locusworks.discord.eighttrack.audio.Mp3UploadResults;
import net.locusworks.discord.eighttrack.database.entities.DiscordGuild;
import net.locusworks.discord.eighttrack.database.entities.GuildSong;
import net.locusworks.discord.eighttrack.database.entities.Song;
import net.locusworks.discord.eighttrack.events.main.AbstractMainEventHandler;
import net.locusworks.discord.eighttrack.handlers.GuildMusicHandler;
import net.locusworks.discord.eighttrack.services.ConfigurationService;
import net.locusworks.discord.eighttrack.services.RepositoryService;
import net.locusworks.logger.ApplicationLogger;
import net.locusworks.logger.ApplicationLoggerFactory;

@Component
public class UploadHandler extends AbstractMainEventHandler {

  private ApplicationLogger logger;
  
  @Autowired
  private Mp3UploadHandler uploadHandler;
  
  @Autowired
  private RepositoryService guildSongRepoService;
  
  @Autowired
  private ConfigurationService confService;
  
  public UploadHandler() {
    super("+upload");
    logger = ApplicationLoggerFactory.getLogger(UploadHandler.class);
  }

  @Override
  public void executeEvent(GuildMusicHandler musicHandler, GuildMessageReceivedEvent event, List<String> commands) {
    if (!isAdmin(event)) return;
    
    for(Attachment attachment : event.getMessage().getAttachments()) {
      attachment.retrieveInputStream().thenAccept((in) -> {
        Mp3UploadResults res = null;
        MessageEmbed embed = null;
        try {

          res = uploadHandler.parse(in);
          if (res.validFile()) {
            embed = persistSong(res, event, attachment.getFileName());
          } else {
            embed = wrongFile(event, attachment.getFileName());
          }

        } catch (Exception ex) {
          logger.error("Unable to get file information: %s", ex.getMessage());
          logger.error(ex);
          embed = error(event, ex, attachment.getFileName()); 
        } finally {
          event.getMessage().delete().queue();
          if (res != null) res.clear();

          if (embed != null) event.getChannel().sendMessage(embed).queue();
        }

      }).exceptionally((err) -> {
        event.getMessage().delete().queue();
        MessageEmbed embed = error(event, err, attachment.getFileName());
        if (embed != null) event.getChannel().sendMessage(embed).queue();
        return null;
      });
    }
  }
  
  private MessageEmbed error(GuildMessageReceivedEvent event, Throwable ex, String fileName) {
    return new EmbedBuilder()
        .setTitle("Unable to upload file: " + fileName)
        .setDescription("There was an error uploading your file: " + ex.getMessage() + ". Please contact admin")
        .setColor(Color.RED)
        .setTimestamp(OffsetDateTime.now())
        .setFooter(event.getGuild().getSelfMember().getEffectiveName(), event.getGuild().getSelfMember().getUser().getAvatarUrl())
        .build();
  }

  private MessageEmbed wrongFile(GuildMessageReceivedEvent event, String fileName) {
    return new EmbedBuilder()
        .setTitle("Invalid File: " + fileName)
        .setDescription("Only music type files are allowed to be uploaded")
        .setColor(Color.RED)
        .setTimestamp(OffsetDateTime.now())
        .setFooter(event.getGuild().getSelfMember().getEffectiveName(), event.getGuild().getSelfMember().getUser().getAvatarUrl())
        .build();
  }

  private MessageEmbed persistSong(Mp3UploadResults result, GuildMessageReceivedEvent event, String fileName) throws Exception {

    DiscordGuild guild = guildSongRepoService.getGuildRepo().findByGuildId(event.getGuild().getIdLong());
    if(guild == null) {
      throw new IOException("Unable to find guild in database. Please contact administrator");
    }

    String hash = HashUtils.hash("SHA-1", result.getData());

    GuildSong gs = guildSongRepoService.getGuildSongRepo().findByGuildAndSongHash(event.getGuild().getIdLong(), hash);
    if (gs != null) {
      MessageEmbed embed = new EmbedBuilder()
          .setColor(Color.ORANGE)
          .setThumbnail(event.getGuild().getIconUrl())
          .setTitle("Upload Results for " + fileName)
          .setDescription(String.format("**%s** by __%s__ already exists. ID: `%s`", gs.getSong().getTitle(), gs.getSong().getArtist(), gs.getUuid()))
          .setTimestamp(OffsetDateTime.now())
          .setFooter(event.getGuild().getSelfMember().getEffectiveName(), event.getGuild().getSelfMember().getUser().getAvatarUrl())
          .build();
      return embed;
    }


    Song song = guildSongRepoService.getSongRepo().findByFileHash(hash);
    if (song == null) {

      Path output = confService.getMusicDirectory();
      if (result.getArtist() != null) {
        output = output.resolve(result.getArtist());
        if (result.getAlbum() != null) {
          output = output.resolve(result.getAlbum());
        }
      }
      output = output.resolve(fileName);

      if (!Files.exists(output.toAbsolutePath().getParent())) {
        Files.createDirectories(output.toAbsolutePath().getParent());
      }

      Files.write(output, result.getData());

      song = new Song();
      song.setAlbum(result.getAlbum());
      song.setArtist(result.getArtist());
      song.setDateAdded(new Date());
      song.setDiscNumber(result.getDiscNumber());
      song.setFileHash(hash);
      song.setFilePath(output.toAbsolutePath().toString());
      song.setGenre(result.getGenre());
      song.setReleaseYear(result.getReleaseDate());
      song.setTitle(result.getTitle());
      song.setTrackNumber(result.getTrackNumber());

      guildSongRepoService.getSongRepo().save(song);
    }

    gs = new GuildSong();
    gs.setDateAdded(new Date());
    gs.setGuild(guild);
    gs.setSong(song);

    String[] uuidArray = UUID.randomUUID().toString().split("-");

    gs.setUuid(uuidArray[uuidArray.length - 1]);

    guildSongRepoService.getGuildSongRepo().save(gs);

    EmbedBuilder builder = new EmbedBuilder()
        .setColor(Color.GREEN)
        .setThumbnail(event.getGuild().getIconUrl())
        .setTitle("Upload Results for " + fileName)
        .setTimestamp(OffsetDateTime.now())
        .setFooter(event.getGuild().getSelfMember().getEffectiveName(), event.getGuild().getSelfMember().getUser().getAvatarUrl())
        .addField("Title", result.getTitle(), true);

    if (!StringUtils.isBlank(result.getArtist()))         builder.addField("Artist", result.getArtist(), true);
    if (!StringUtils.isBlank(result.getAlbum()))          builder.addField("Album", result.getAlbum(), true);
    if (!StringUtils.isBlank(result.getTrackNumber()))    builder.addField("Track", result.getTrackNumber(), true);
    if (!StringUtils.isBlank(result.getDiscNumber()))     builder.addField("Disc", result.getDiscNumber(), true);
    if (!StringUtils.isBlank(result.getReleaseDate()))    builder.addField("Year", result.getReleaseDate(), true);

    builder.addField("ID", gs.getUuid(), true);

    MessageEmbed embed = builder.build();

    try {
      guildSongRepoService.logEntry(event.getGuild().getIdLong(), "%s uploaded", result);
    } catch (Exception ex) {}

    return embed;
  }

  @Override
  public String help() {
    return "upload song";
  }

}
